@@ -1,6 +1,6 @@
 /***********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2009, International Business Machines Corporation
+ * Copyright (c) 1997-2010, International Business Machines Corporation
  * and others. All Rights Reserved.
  ***********************************************************************/
 
@@ -18,7 +18,7 @@
 
 #define CHECK(status, msg) \
     if (U_FAILURE(status)) { \
-      errcheckln(status, (UnicodeString(u_errorName(status)) + UnicodeString(" : " ) )+ msg); \
+      dataerrln((UnicodeString(u_errorName(status)) + UnicodeString(" : " ) )+ msg); \
         return; \
     }
 
@@ -531,21 +531,7 @@ void IntlCalendarTest::TestJapaneseFormat() {
 
     // Test parse with incomplete information
     fmt = new SimpleDateFormat(UnicodeString("G y"), Locale("en_US@calendar=japanese"), status);
-    /* The test data below should points to 1868-09-08T00:00:00 in America/Los_Angeles.
-     * The time calculated by original test code uses -7:00 UTC offset, because it assumes
-     * DST is observed (because of a timezone bug, DST is observed for early 20th century
-     * day to infinite past time).  The bug was fixed and DST is no longer used for time before
-     * 1900 for any zones.  However, ICU timezone transition data is represented by 32-bit integer
-     * (sec) and cannot represent transitions before 1901 defined in Olson tzdata.  For example,
-     * based on Olson definition, offset -7:52:58 should be used for Nov 18, 1883 or older dates.
-     * If ICU properly capture entire Olson zone definition, the start time of "Meiji 1" is
-     * -3197117222000. -Yoshito
-     */
-    /* TODO: When ICU support the Olson LMT offset for America/Los_Angeles, we need to update
-     * the reference data.
-     */
-    //aDate = -3197120400000.;
-    aDate = -3197116800000.;
+    aDate = -3197117222000.0;
     CHECK(status, "creating date format instance");
     if(!fmt) { 
         errln("Coudln't create en_US instance");
@@ -597,8 +583,7 @@ void IntlCalendarTest::TestJapaneseFormat() {
     }
     {
         UnicodeString expect = CharsToUnicodeString("\\u5b89\\u6c385\\u5e747\\u67084\\u65e5\\u6728\\u66dc\\u65e5");
-        //UDate         expectDate = -6106035600000.0;
-        UDate         expectDate = -6106032000000.0; // 1776-07-04T00:00:00Z-0800
+        UDate         expectDate = -6106032422000.0; // 1776-07-04T00:00:00Z-075258
         Locale        loc("ja_JP@calendar=japanese");
         
         status = U_ZERO_ERROR;
@@ -616,9 +601,7 @@ void IntlCalendarTest::TestJapaneseFormat() {
     }
     {   // This Feb 29th falls on a leap year by gregorian year, but not by Japanese year.
         UnicodeString expect = CharsToUnicodeString("\\u5EB7\\u6B632\\u5e742\\u670829\\u65e5\\u65e5\\u66dc\\u65e5");
-        // Add -1:00 to the following for historical TZ - aliu
-        //UDate         expectDate =  -16214403600000.0;  // courtesy of date format round trip test
-        UDate         expectDate =  -16214400000000.0;  // 1456-03-09T00:00:00Z-0800
+        UDate         expectDate =  -16214400422000.0;  // 1456-03-09T00:00Z-075258
         Locale        loc("ja_JP@calendar=japanese");
         
         status = U_ZERO_ERROR;
