@@ -79,6 +79,45 @@ void WPS4Parser::parse(WPXDocumentInterface *documentInterface)
 WPS4Parser private
 */
 
+static const struct _fmap {
+	const char *name;
+	const char *repl;
+	int codepage;
+} font_map_s[] = {
+	// Built-in Windows
+	{"Arial CYR","Arial",1251},
+	{"Arial CE","Arial",1250},
+	{"Courier New CYR","Courier New",1251},
+	{"Courier New CE","Courier New",1250},
+	{"Times New Roman CYR","Times New Roman",1251},
+	{"Times New Roman CE","Times New Roman",1250},
+
+	// Custom
+	{"Baltica",NULL,1251},
+	{"Pragmatica",NULL,1251}
+};
+
+#ifdef _MSC_VER
+#define strcasecmp _strcmpi
+#endif
+
+/**
+Maps some of legacy fonts to modern ones and set encoding accordingly.
+\param[in,out] str May contain mapped font name
+*/
+
+int getCodepage(std::string &str)
+{
+	const char *s = str.c_str();
+	for (unsigned i=0; i < (sizeof(font_map_s)/sizeof(_fmap)); i++)
+	{
+		if (!strcasecmp(s,font_map_s[i].name)) {
+			if (font_map_s[i].repl) str = font_map_s[i].repl;
+			return font_map_s[i].codepage;
+		}
+	}
+	return 0;
+}
 
 /**
  * Reads fonts table into memory.
@@ -130,7 +169,10 @@ void WPS4Parser::readFontsTable(WPXInputStream * input)
 		WPS_DEBUG_MSG(("Works: info: count=%i, font_number=%i, unknown=%i, name=%s\n",
 			 fonts.size(), font_number, unknown_byte, s.c_str()));
 		s.append(1, (char)0);
-		fonts[font_number] = s;
+		wpsfont f;
+		f.cp=getCodepage(s);
+		f.name=s;
+		fonts[font_number] = f;
 	}
 }
 
@@ -324,6 +366,28 @@ const char * WPS2FontNameFromIndex(uint8_t font_n)
 	throw ParseException();
 }
 
+// TODO: Verify exact values
+const uint32_t WPS4_Color[]=
+{
+	// 0x00RRGGBB
+	0, // auto
+	0,
+	0x0000FF,
+	0x00FFFF,
+	0x00FF00,
+	0xFF00FF,
+	0xFF0000,
+	0xFFFF00,
+	0x808080,
+	0xFFFFFF,
+	0x000080,
+	0x008080,
+	0x008000,
+	0x800080,
+	0x808000,
+	0xC0C0C0
+};
+
 /**
  * Process a character property change.  The Works format supplies
  * all the character formatting each time there is any change (as
@@ -333,13 +397,16 @@ const char * WPS2FontNameFromIndex(uint8_t font_n)
  */
 void WPS4Parser::propertyChange(std::string rgchProp, WPS4ContentListener *listener)
 {
+	/* set default properties */
+	uint32_t textAttributeBits = 0;
+	uint8_t  colorid = 0;
+
+	listener->setFontSize(12);
+	
 	if (0 == rgchProp.length())
 		return;
 
 //fixme: this method is immature
-	/* set default properties */
-	uint32_t textAttributeBits = 0;
-	
 	/* set difference from default properties */
 	if (rgchProp[0] & 0x01)
 	{
@@ -358,7 +425,7 @@ void WPS4Parser::propertyChange(std::string rgchProp, WPS4ContentListener *liste
 		/* text font */
 		uint8_t font_n = (uint8_t)rgchProp[2];
 
-		if (3 == getHeader()->getMajorVersion())
+		if (3 <= getHeader()->getMajorVersion())
 		{
 			if (fonts.find(font_n) == fonts.end())
 			{
@@ -367,7 +434,10 @@ void WPS4Parser::propertyChange(std::string rgchProp, WPS4ContentListener *liste
 				throw ParseException();
 			}
 			else
-				listener->setTextFont(fonts[font_n].c_str());
+			{
+				listener->setTextFont(fonts[font_n].name.c_str());
+				listener->setCodepage(fonts[font_n].cp);
+			}
 		}
 		if (2 == getHeader()->getMajorVersion())
 		{
@@ -383,11 +453,6 @@ void WPS4Parser::propertyChange(std::string rgchProp, WPS4ContentListener *liste
 	{
 		listener->setFontSize(((uint8_t)rgchProp[4])/2);
 	}
-	else
-	{
-		/* default font size */
-		listener->setFontSize(12);
-	}
 	if (rgchProp.length() >= 6)
 	{
 		WPS_DEBUG_MSG(("rgchProp[1] & 0x40 = %i, rgchProp[5]=%i\n", (rgchProp[1] & 0x40), (uint8_t)rgchProp[5]));
@@ -401,8 +466,125 @@ void WPS4Parser::propertyChange(std::string rgchProp, WPS4ContentListener *liste
 			textAttributeBits |= WPS_SUBSCRIPT_BIT;					
 		}
 	}
+	if (rgchProp.length() >= 8) {
+		colorid = (uint8_t)rgchProp[7] & 0xF;
+	}
 	propertyChangeDelta(textAttributeBits, listener);
+	listener->setColor(WPS4_Color[colorid]);
 }
+
+void WPS4Parser::propertyChangePara(std::string rgchProp, WPS4ContentListener *listener)
+{
+	static const uint8_t _align[]={WPS_PARAGRAPH_JUSTIFICATION_LEFT,
+		WPS_PARAGRAPH_JUSTIFICATION_CENTER,WPS_PARAGRAPH_JUSTIFICATION_RIGHT,
+		WPS_PARAGRAPH_JUSTIFICATION_FULL};
+
+	int pindent = 0, pbefore = 0, pafter = 0;
+	int pleft =0, pright =0;
+
+	std::vector<TabPos> tabs;
+	listener->setTabs(tabs);
+
+	int pf = 0;
+
+	if (4 > rgchProp.length()) return;
+
+	unsigned x = 3;
+
+	while (x < rgchProp.length()) {
+		char code = rgchProp[x++];
+
+		switch (code) {
+			case 0x5:	// alignment
+				{
+					uint8_t a = rgchProp[x];
+					if (a < 4)
+						listener->setAlign(_align[a]);
+					x++;
+				}
+				break;
+			case 0x7:
+				pf |= WPS_PARAGRAPH_LAYOUT_NO_BREAK;
+				x++;
+				break;
+			case 0x8:
+				pf |= WPS_PARAGRAPH_LAYOUT_WITH_NEXT;
+				x++;
+				break;
+			case 0x11:
+				pleft = WPS_LE_GET_GUINT16(&rgchProp.c_str()[x]);
+				x+=2;
+				break;
+			case 0x12:
+				pright = WPS_LE_GET_GUINT16(&rgchProp.c_str()[x]);
+				x+=2;
+				break;
+			case 0x14:
+				pindent = (int16_t) WPS_LE_GET_GUINT16(&rgchProp.c_str()[x]);
+				x+=2;
+				break;
+			case 0x16:
+				pbefore = WPS_LE_GET_GUINT16(&rgchProp.c_str()[x]);
+				x+=2;
+				break;
+			case 0x17:
+				pafter = WPS_LE_GET_GUINT16(&rgchProp.c_str()[x]);
+				x+=2;
+				break;
+			/*
+				case 0x9:
+				case 0xA:
+				case 0xB:
+				case 0xC:
+				case 0xD:
+				case 0xE:
+				case 0x18:
+				case 0x1b:
+					// noop, skip 1
+					c+=2;
+					break;
+				case 0x15:
+					// noop, skip 2
+					c+=3;
+					break;
+			*/
+			case 0xF:
+				{
+					const char *td = rgchProp.c_str();
+					uint16_t count;
+					TabPos pos;
+
+					count = WPS_LE_GET_GUINT16(&td[x+2]);
+					if (count <= 20) {
+						for (int i=0; i<count; i++) {
+							char v;
+							pos.pos = WPS_LE_GET_GUINT16(&td[x+4+i*2]) / 1440.0;
+							v = td[x+4+count*2+i];
+							switch (v&3) {
+							case 1: pos.align = WPS_TAB_CENTER; break;
+							case 2: pos.align = WPS_TAB_RIGHT; break;
+							case 3: pos.align = WPS_TAB_DECIMAL; break;
+							}
+							// TODO: leader
+							tabs.push_back(pos);
+						}
+						listener->setTabs(tabs);
+					}
+					x+=rgchProp[x]+(rgchProp[x+1]<<8)+1;
+				}
+				break;
+			default:
+				WPS_DEBUG_MSG(("Unknown parafmt code: %02X\n",code));
+				break;
+		}
+	}
+
+	if (pf != 0) listener->setParaFlags(pf);
+
+	listener->setMargins(pindent/1440.0,pleft/1440.0,pright/1440.0,
+		pbefore/1440.0,pafter/1440.0);
+}
+
 /**
  * Take a character in CP850 encoding, convert it
  * and append it to the text buffer as UTF8.
@@ -437,37 +619,7 @@ void WPS4Parser::appendCP850(const uint8_t readVal, WPS4ContentListener *listene
 		ucs4Character = (uint32_t) cp850toUCS4[readVal - 0x80];
 	}
 
-	uint8_t first;
-	int len;
-	if (ucs4Character < 0x80) {
-		first = 0; len = 1;
-	}
-	else if (ucs4Character < 0x800) {
-		first = 0xc0; len = 2;
-	}
-	else if (ucs4Character < 0x10000) {
-		first = 0xe0; len = 3;
-	}
-	else if (ucs4Character < 0x200000) {
-		first = 0xf0; len = 4;
-	}
-	else if (ucs4Character < 0x4000000) {
-		first = 0xf8; len = 5;
-	}
-	else {
-		first = 0xfc; len = 6;
-	}
-
-	uint8_t outbuf[6] = { 0, 0, 0, 0, 0, 0 };
-	int i;
-	for (i = len - 1; i > 0; --i) {
-		outbuf[i] = (ucs4Character & 0x3f) | 0x80;
-		ucs4Character >>= 6;
-	}
-	outbuf[0] = ucs4Character | first;
-
-	for (i = 0; i < len; i++)
-		listener->insertCharacter(outbuf[i]);
+	appendUCS(ucs4Character,listener);
 }
 
 /**
@@ -494,6 +646,87 @@ void WPS4Parser::appendCP1252(const uint8_t readVal, WPS4ContentListener *listen
 			return;
 	}
 
+	appendUCS(ucs4Character,listener);
+}
+
+uint32_t s_CP1250(const uint8_t readVal)
+{
+	static const uint16_t cp1250toUCS4[128] = {
+		0x20ac, 0xfffd, 0x201a, 0xfffd, 0x201e, 0x2026, 0x2020, 0x2021,
+		0xfffd, 0x2030, 0x0160, 0x2039, 0x015a, 0x0164, 0x017d, 0x0179,
+		0xfffd, 0x2018, 0x2019, 0x201c, 0x201d, 0x2022, 0x2013, 0x2014,
+		0xfffd, 0x2122, 0x0161, 0x203a, 0x015b, 0x0165, 0x017e, 0x017a, 
+		0x00a0, 0x02c7, 0x02d8, 0x0141, 0x00a4, 0x0104, 0x00a6, 0x00a7,
+		0x00a8, 0x00a9, 0x015e, 0x00ab, 0x00ac, 0x00ad, 0x00ae, 0x017b, 
+		0x00b0, 0x00b1, 0x02db, 0x0142, 0x00b4, 0x00b5, 0x00b6, 0x00b7,
+		0x00b8, 0x0105, 0x015f, 0x00bb, 0x013d, 0x02dd, 0x013e, 0x017c,
+
+		0x0154, 0x00c1, 0x00c2, 0x0453, 0x201e, 0x2026, 0x2020, 0x2021,
+		0x20ac, 0x2030, 0x0409, 0x2039, 0x040a, 0x040c, 0x040b, 0x040f,
+		0x0452, 0x2018, 0x2019, 0x201c, 0x201d, 0x2022, 0x2013, 0x2014,
+		0xfffd, 0x2122, 0x0459, 0x203a, 0x045a, 0x045c, 0x045b, 0x045f, 
+		0x0155, 0x00e1, 0x00e2, 0x0103, 0x00e4, 0x013a, 0x0107, 0x00e7,
+		0x010d, 0x00e9, 0x0119, 0x00eb, 0x011b, 0x00ed, 0x00ee, 0x010f, 
+		0x0111, 0x0144, 0x0148, 0x00f3, 0x00f4, 0x0151, 0x00f6, 0x00b7,
+		0x0159, 0x016f, 0x00fa, 0x0171, 0x00fc, 0x00fd, 0x0163, 0x02d9 };
+
+	uint32_t ucs4Character = 0;
+	if (readVal < 0x80) {
+		ucs4Character = (uint32_t) readVal;
+	} else {
+		ucs4Character = (uint32_t) cp1250toUCS4[readVal - 0x80];
+	}
+
+	return ucs4Character;
+}
+
+uint32_t s_CP1251(const uint8_t readVal)
+{
+	static const uint16_t cp1251toUCS4[64] = {
+		0x0402, 0x0403, 0x201a, 0x0453, 0x201e, 0x2026, 0x2020, 0x2021,
+		0x20ac, 0x2030, 0x0409, 0x2039, 0x040a, 0x040c, 0x040b, 0x040f,
+		0x0452, 0x2018, 0x2019, 0x201c, 0x201d, 0x2022, 0x2013, 0x2014,
+		0xfffd, 0x2122, 0x0459, 0x203a, 0x045a, 0x045c, 0x045b, 0x045f, 
+		0x00a0, 0x040e, 0x045e, 0x0408, 0x00a4, 0x0490, 0x00a6, 0x00a7,
+		0x0401, 0x00a9, 0x0404, 0x00ab, 0x00ac, 0x00ad, 0x00ae, 0x0407, 
+		0x00b0, 0x00b1, 0x0406, 0x0456, 0x0491, 0x00b5, 0x00b6, 0x00b7,
+		0x0451, 0x2116, 0x0454, 0x00bb, 0x0458, 0x0405, 0x0455, 0x0457 };
+
+	uint32_t ucs4Character = 0;
+	if (readVal < 0x80) {
+		ucs4Character = (uint32_t) readVal;
+	} else if (readVal >= 0xc0) {
+		ucs4Character = (uint32_t) readVal+0x410-0xc0;
+	} else {
+		ucs4Character = (uint32_t) cp1251toUCS4[readVal - 0x80];
+	}
+
+	return ucs4Character;
+}
+
+void WPS4Parser::appendCP(const uint8_t readVal, int codepage, WPS4ContentListener *listener)
+{
+	if (!codepage) codepage=listener->getCodepage();
+	if (codepage == 1251) {
+		appendUCS(s_CP1251(readVal),listener);
+		return;
+	} else if (codepage == 1250) {
+		appendUCS(s_CP1250(readVal),listener);
+		return;
+	} else if (codepage == 850) {
+		appendCP850(readVal,listener);
+		return;
+	}
+	appendCP1252(readVal,listener);
+}
+
+void WPS4Parser::appendUCS(const uint16_t readVal, WPS4ContentListener *listener)
+{
+	uint32_t ucs4Character = (uint32_t) readVal;
+	if (ucs4Character == 0xfffd)
+//		throw GenericException();
+		return;
+
 	uint8_t first;
 	int len;
 	if (ucs4Character < 0x80) {
@@ -527,6 +760,7 @@ void WPS4Parser::appendCP1252(const uint8_t readVal, WPS4ContentListener *listen
 		listener->insertCharacter(outbuf[i]);
 }
 
+
 /**
  * Read the text of the document using previously-read
  * formatting information.
@@ -536,6 +770,7 @@ void WPS4Parser::readText(WPXInputStream * input, WPS4ContentListener *listener)
 {
 	oldTextAttributeBits = 0;
 	std::vector<FOD>::iterator FODs_iter;	
+	std::vector<FOD>::iterator PFOD_iter;
 #if 1
 	/* dump for debugging */
 	for (FODs_iter = CHFODs.begin(); FODs_iter!= CHFODs.end(); FODs_iter++)
@@ -546,70 +781,90 @@ void WPS4Parser::readText(WPXInputStream * input, WPS4ContentListener *listener)
 #endif
 
 	uint32_t last_fcLim = 0x100;
+	PFOD_iter = PAFODs.begin();
 	for (FODs_iter = CHFODs.begin(); FODs_iter!= CHFODs.end(); FODs_iter++)
 	{
-		uint32_t len = (*FODs_iter).fcLim - last_fcLim;
-		WPS_DEBUG_MSG(("Works: info: txt len=%02i rgchProp=%s\n", 
-			len, to_bits((*FODs_iter).fprop.rgchProp).c_str()));
-		if ((*FODs_iter).fprop.cch > 0)
-			propertyChange((*FODs_iter).fprop.rgchProp, listener);
-		input->seek(last_fcLim, WPX_SEEK_SET);			
-		for (uint32_t i = len; i>0; i--)
-		{
-			uint8_t readVal = readU8(input);
-			if (0x00 == readVal)
-				break;
-				
-			switch (readVal)
+		uint32_t len1 = (*FODs_iter).fcLim - last_fcLim;
+		do {
+			uint32_t len2 = (*PFOD_iter).fcLim - last_fcLim;
+			uint32_t len = len1;
+			if (len2 < len) len=len2;
+			WPS_DEBUG_MSG(("Works: info: txt len=%02i rgchProp=%s\n", 
+				len, to_bits((*FODs_iter).fprop.rgchProp).c_str()));
+
+			if ((*FODs_iter).fprop.cch > 0)
+				propertyChange((*FODs_iter).fprop.rgchProp, listener);
+			if ((*PFOD_iter).fprop.cch > 0)
+				propertyChangePara((*PFOD_iter).fprop.rgchProp, listener);
+			input->seek(last_fcLim, WPX_SEEK_SET);			
+			for (uint32_t i = len; i>0; i--)
 			{
-				case 0x01:
-				case 0x02:
-				case 0x03:
-				case 0x04:
-				case 0x05:
-				case 0x06:
-				case 0x07:
-				case 0x08:
-				case 0x09:
-				case 0x0A:
-				case 0x0B:
-				case 0x0E:
-				case 0x0F:
-				case 0x10:
-				case 0x11:
-				case 0x12:
-				case 0x13:
-				case 0x14:
-				case 0x15:
-				case 0x16:
-				case 0x17:
-				case 0x18:
-				case 0x19:
-				case 0x1A:
-				case 0x1B:
-				case 0x1C:
-				case 0x1D:
-				case 0x1E:
-				case 0x1F:
+				uint8_t readVal = readU8(input);
+				if (0x00 == readVal)
 					break;
+					
+				switch (readVal)
+				{
+					case 0x01:
+					case 0x03:
+					case 0x04:
+					case 0x05:
+					case 0x06:
+					case 0x07:
+					case 0x08:
+					case 0x09:
+					case 0x0A:
+					case 0x0B:
+					case 0x0E:
+					case 0x0F:
+					case 0x10:
+					case 0x11:
+					case 0x12:
+					case 0x13:
+					case 0x14:
+					case 0x15:
+					case 0x16:
+					case 0x17:
+					case 0x18:
+					case 0x19:
+					case 0x1A:
+					case 0x1B:
+					case 0x1C:
+					case 0x1D:
+					case 0x1E:
+					case 0x1F:
+						break;
 
-				case 0x0C:
-					listener->insertBreak(WPS_PAGE_BREAK);
-					break;
+					case 0x02:
+						// TODO: check special bit
+						listener->setFieldType(WPS_FIELD_PAGE);
+						listener->insertField();
+						break;
 
-				case 0x0D:
-					listener->insertEOL();
-					break;
+					case 0x0C:
+						listener->insertBreak(WPS_PAGE_BREAK);
+						break;
 
-				default:
-					if (m_worksVersion == 2)
-						this->appendCP850(readVal, listener);
-					else
-						this->appendCP1252(readVal, listener);
-					break;
+					case 0x0D:
+						listener->insertEOL();
+						break;
+
+					default:
+						if (m_worksVersion == 2)
+							this->appendCP(readVal, 850, listener);
+						else
+							this->appendCP(readVal, 0, listener);
+						break;
+				}
 			}
-		}	
-		last_fcLim = (*FODs_iter).fcLim;	
+
+			len1 -= len;
+			len2 -= len;
+
+			if (len2 == 0) PFOD_iter++;
+
+			last_fcLim += len;	
+		} while (len1 > 0);
 	}
 }
 
@@ -624,15 +879,15 @@ void WPS4Parser::parsePages(std::list<WPSPageSpan> &pageList, WPXInputStream *in
 	/* read page format */
 	input->seek(0x64, WPX_SEEK_SET);
 	uint16_t margin_top = readU16(input);
-	input->seek(0x66, WPX_SEEK_SET);
+	//input->seek(0x66, WPX_SEEK_SET);
 	uint16_t margin_bottom = readU16(input);
-	input->seek(0x68, WPX_SEEK_SET);
+	//input->seek(0x68, WPX_SEEK_SET);
 	uint16_t margin_left = readU16(input);
-	input->seek(0x6A, WPX_SEEK_SET);
+	//input->seek(0x6A, WPX_SEEK_SET);
 	uint16_t margin_right = readU16(input);
-	input->seek(0x6C, WPX_SEEK_SET);
+	//input->seek(0x6C, WPX_SEEK_SET);
 	uint16_t page_height = readU16(input);
-	input->seek(0x6E, WPX_SEEK_SET);
+	//input->seek(0x6E, WPX_SEEK_SET);
 	uint16_t page_width = readU16(input);
 	input->seek(0x7A, WPX_SEEK_SET);
 	uint8_t page_orientation = readU8(input);
@@ -789,3 +1044,8 @@ void WPS4ContentListener::attributeChange(const bool isOn, const uint8_t attribu
 	else
 		m_ps->m_textAttributeBits ^= textAttributeBit;
 }
+
+int WPS4ContentListener::getCodepage()
+{
+	return m_ps->m_codepage;
+}
